<?php

/**
 * @package        Joomla
 * @subpackage     Membership Pro
 * @author         Tuan Pham Ngoc
 * @copyright      Copyright (C) 2012 - 2023 Ossolution Team
 * @license        GNU/GPL, see LICENSE.php
 */
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Table\Table;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;

class plgOSMembershipSubscriptionControl extends CMSPlugin
{
	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 */
	protected $app;

	/**
	 * Database object.
	 *
	 * @var    JDatabaseDriver
	 */
	protected $db;

	/**
	 * Make language files will be loaded automatically.
	 *
	 * @var bool
	 */
	protected $autoloadLanguage = true;

	/**
	 * Render settings from
	 *
	 * @param   OSMembershipTablePlan  $row
	 *
	 * @return array
	 */
	public function onEditSubscriptionPlan($row)
	{
		if (!$this->isExecutable())
		{
			return [];
		}

		ob_start();
		$this->drawSettingForm($row);
		$form = ob_get_contents();
		ob_end_clean();

		return [
			'title' => Text::_('PLG_OSMEMBERSHIP_SUBSCRIPTION_CONTROL_SETTINGS'),
			'form'  => $form,
		];
	}

	/**
	 * Store setting into database
	 *
	 * @param   OSMembershipTablePlan  $row
	 * @param   Boolean                $isNew  true if create new plan, false if edit
	 */
	public function onAfterSaveSubscriptionPlan($context, $row, $data, $isNew)
	{
		if (!$this->isExecutable())
		{
			return;
		}

		$params = new Registry($row->params);

		$keys = ['expiring_plan_ids', 'subscription_expired_subscribe_plan_ids'];

		foreach ($keys as $key)
		{
			$params->set($key, implode(',', $data[$key] ?? []));
		}

		$row->params = $params->toString();

		$row->store();
	}

	/**
	 * Run when a membership activated
	 *
	 * @param   OSMembershipTableSubscriber  $row
	 */
	public function onMembershipActive($row)
	{
		if (!$row->user_id)
		{
			return;
		}

		$plan = Table::getInstance('Plan', 'OSMembershipTable');
		$plan->load($row->plan_id);
		$params          = new Registry($plan->params);
		$expiringPlanIds = array_filter(ArrayHelper::toInteger(explode(',', $params->get('expiring_plan_ids', ''))));

		if (count($expiringPlanIds) === 0)
		{
			return;
		}

		// Find all active and pending subscriptions of the current user for these plans and expiring it
		$db    = $this->db;
		$query = $db->getQuery(true)
			->select('id')
			->from('#__osmembership_subscribers')
			->where('published IN (0, 1)')
			->where('user_id = ' . $row->user_id)
			->where('plan_id IN (' . implode(',', $expiringPlanIds) . ')');
		$db->setQuery($query);
		$ids = $db->loadColumn();

		if (count($ids) === 0)
		{
			return;
		}

		PluginHelper::importPlugin('osmembership');

		/* @var OSMembershipTableSubscriber $rowSubscription */
		$app             = Factory::getApplication();
		$rowSubscription = Table::getInstance('Subscriber', 'OSMembershipTable');
		$now             = Factory::getDate()->toSql();

		foreach ($ids as $id)
		{
			$rowSubscription->load($id);
			$rowSubscription->to_date              = $now;
			$rowSubscription->published            = 2;
			$rowSubscription->first_reminder_sent  = 1;
			$rowSubscription->second_reminder_sent = 1;
			$rowSubscription->third_reminder_sent  = 1;
			$rowSubscription->store();

			//Trigger plugins
			$app->triggerEvent('onMembershipExpire', [$rowSubscription]);
		}
	}

	/**
	 * Run when a membership expiried die
	 *
	 * @param   OSMembershipTableSubscriber  $row
	 */
	public function onMembershipExpire($row)
	{
		if (!$row->user_id)
		{
			return;
		}

		$plan = Table::getInstance('Plan', 'OSMembershipTable');
		$plan->load($row->plan_id);
		$params           = new Registry($plan->params);
		$subscribePlanIds = array_filter(ArrayHelper::toInteger(explode(',', $params->get('subscription_expired_subscribe_plan_ids', ''))));

		if (count($subscribePlanIds) === 0)
		{
			return;
		}

		// Get data of the current subscription
		/* @var OSMembershipModelApi $model */
		$model = MPFModel::getTempInstance('Api', 'OSMembershipModel');
		$data = $model->getSubscriptionData($row->id);

		foreach ($subscribePlanIds as $planId)
		{
			$subscriptionData = $data;
			$subscriptionData['plan_id'] = $planId;
			unset($subscriptionData['act']);

			try
			{
				$model->store($data);
			}
			catch (Exception $e)
			{

			}
		}

	}

	/**
	 * Method to check if the plugin is executable
	 *
	 * @return bool
	 */
	private function isExecutable()
	{
		if ($this->app->isClient('site') && !$this->params->get('show_on_frontend'))
		{
			return false;
		}

		return true;
	}

	/**
	 * Display form allows users to change setting for this subscription plan
	 *
	 * @param   OSMembershipTablePlan  $row
	 */
	private function drawSettingForm($row)
	{
		$db      = $this->db;
		$query   = $db->getQuery(true)
			->select('*')
			->from('#__osmembership_plans')
			->where('published = 1')
			->order('ordering');
		$options = [];

		foreach ($db->loadObjectList() as $plan)
		{
			$options[] = HTMLHelper::_('select.option', $plan->id, $plan->title);
		}

		require PluginHelper::getLayoutPath($this->_type, $this->_name, 'form');
	}
}
